#region Using directives
using System;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;
using System.Text;
using Microsoft.Win32;
using System.Security.Permissions;
using System.Drawing;
using System.Windows.Forms;
using System.Diagnostics;
#endregion

namespace ShellExt
{
    #region VisualStudioToolsExtension
    [Guid("34F6A966-58D5-41a6-8BD7-64F65267AF2E")]
    public class VisualStudioToolsExtension : ContextMenuShellExtension
    {
        // Redundant guid entry. The GUID attribute, applied to this class, cannot be read 
        // from within the STATIC registerServer en UnregisterServer methods. 
        const string guid = "{34F6A966-58D5-41a6-8BD7-64F65267AF2E}";

        #region Event handlers
        /// <summary>
        /// Executes the command menu item.
        /// </summary>
        /// <param name="sender">The menu item that is executed.</param>
        /// <param name="ea">The <see cref="ShellExt.ExecuteItemEventArgs"/>Instance containing the event data.</param>
        public void ExecuteCommandMenuItem(object sender, ExecuteItemEventArgs ea)
        {
            try
            {
                // Get the directory name
                StringBuilder sb = new StringBuilder(1024);
                Helpers.DragQueryFile(m_hDrop, 0, sb, sb.Capacity + 1);
                string file = sb.ToString();
                ContextMenuItem mi = (ContextMenuItem)sender;

                Process process = new Process();
                process.StartInfo.FileName = mi.Properties["filename"];
                process.StartInfo.Arguments = mi.Properties["arguments"] + " \"" + file + " \"";
                
                process.Start();
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
            }
        }

        /// <summary>
        /// Draw menu item
        /// </summary>
        /// <param name="sender">The menuitem that is to be drawn</param>
        /// <param name="ea">Instance containing the event data.</param>
        public void DrawMenuItem(object sender, DrawItemEventArgs ea)
        {
            int x = ea.Bounds.Left;
            int y = ea.Bounds.Top;

            // Perform default highlight and normal background painting
            ea.DrawBackground();

            Stream imgStream = null;
            Bitmap bmp = null;
            // get a reference to the current assembly
            Assembly a = Assembly.GetExecutingAssembly();

            // get the command bitmap as a stream from the assembly, prefix with default project namespace
            ContextMenuItem mi = (ContextMenuItem)sender;
            imgStream = a.GetManifestResourceStream(this.GetType(), mi.Properties["resourcename"]);

            if (!(imgStream == null))
            {
                // create a new bitmap from the stream
                bmp = Bitmap.FromStream(imgStream) as Bitmap;
                if (!(bmp == null))
                {
                    // Make the default transparent color transparent for this Bitmap object
                    bmp.MakeTransparent();
                    // Draw Bitmap to the context menu, start 17 pixels from the right and draw the 16 pixels wide image
                    ea.Graphics.DrawImage(bmp, new Point(ea.Bounds.Right - 17, ea.Bounds.Top));
                }
                // Tidy up image stream
                imgStream.Close();
            }

            // Add the standard image width to the horizontal offset so that menuitem is aligned with others
            x += SystemInformation.MenuCheckSize.Width + 1;
            // Adjust y offset by 1
            y += 1;

            // Create a bold version of the currently selected font
            Font boldFont = new Font(ea.Font.FontFamily, ea.Font.SizeInPoints, FontStyle.Bold);
            ea.Graphics.DrawString(Items[ea.Index - 1].Text, boldFont, new SolidBrush(ea.ForeColor), x, y);
        } // DrawMenuItem(sender, ea)

        #endregion

        #region Overrides

        /// <summary>
        /// Accepts the context of the popup for this extension
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <returns>
        /// True in case the extension should be activated otherwise false
        /// </returns>
        protected override bool AcceptContext(string filePath)
        {
            return true;
        }

        /// <summary>
        /// Populates the menu with menuitems
        /// </summary>
        protected override void InitializeMenu()
        {
            // Now retrieve the menu information from the registry
            RegistryKey sc = Registry.LocalMachine;
            sc = sc.OpenSubKey("Software\\VisualStudioTools", true);
            if (sc.SubKeyCount > 0)
            {
                // Loop through all the subkeys and create a menuitem for each
                foreach (string name in sc.GetSubKeyNames())
                {
                    try
                    {
                        RegistryKey item = sc.OpenSubKey(name, true);
                        string text = (string)item.GetValue("");
                        string filename = (string)item.GetValue("filename");
                        string arguments = (string)item.GetValue("arguments");
                        string icon = (string)item.GetValue("icon");

                        ContextMenuItem m = new ContextMenuItem();
                        m.Text = text;
                        // Dynamic storage of data, to be used in later events
                        m.Properties["filename"] = filename;
                        m.Properties["arguments"] = arguments;
                        m.Properties["resourcename"] = icon;

                        m.OnExecuteItem += ExecuteCommandMenuItem;
                        m.OnDrawItem += DrawMenuItem;
                        // m.OnMeasureItem += MeasureMenuItem;
                        m.OwnerDraw = true;
                        Items.Add(m);
                    }
                    catch (Exception)
                    {
                    }
                }
                // Items are added to popup in reverse order, reverse the items now to compensatie for this.
                Items.Reverse();
            }
        }
        #endregion

        #region Registration
        [System.Runtime.InteropServices.ComRegisterFunctionAttribute()]
        static void RegisterServer(String str1)
        {
            try
            {
                RegistryKey root;
                RegistryKey rk;
                root = Registry.CurrentUser;
                rk = root.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer", true);
                rk.SetValue("DesktopProcess", 1);
                rk.Close();

                // Mark this extension as an approved shell extension
                root = Registry.LocalMachine;
                rk = root.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved", true);
                rk.SetValue(guid.ToString(), "Visual Studio Tools extension");
                rk.Close();

                // Set "exefile\\shellex\\ContextMenuHandlers\\VisualStudioTools" regkey to the guid
                root = Registry.ClassesRoot;
                rk = root.CreateSubKey("exefile\\shellex\\ContextMenuHandlers\\VisualStudioTools");
                rk.SetValue("", guid.ToString());
                rk.Close();

                // Add setting for menu items
                root = Registry.LocalMachine;
                rk = root.CreateSubKey("Software\\VisualStudioTools");
                rk = root.CreateSubKey("Software\\VisualStudioTools\\Install Service");
                rk.SetValue("", "Install service");
                rk.SetValue("filename", "C:\\WINDOWS\\Microsoft.NET\\Framework\\v2.0.50727\\installutil.exe");
                rk.SetValue("arguments", "");
                rk.SetValue("icon", "installservice.ico");

                rk = root.CreateSubKey("Software\\VisualStudioTools\\Uninstall Service");
                rk.SetValue("", "Uninstall service");
                rk.SetValue("filename", "C:\\WINDOWS\\Microsoft.NET\\Framework\\v2.0.50727\\installutil.exe");
                rk.SetValue("arguments", "-u");
                rk.SetValue("icon", "uninstallservice.ico");
               
                rk.Close();
            }
            catch (Exception e)
            {
                System.Console.WriteLine(e.ToString());
            }
        }

        [System.Runtime.InteropServices.ComUnregisterFunctionAttribute()]
        static void UnregisterServer(String str1)
        {
            try
            {
                RegistryKey root;
                RegistryKey rk;

                // Remove ShellExtenstions registration
                root = Registry.LocalMachine;
                rk = root.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved", true);
                rk.DeleteValue(guid);
                rk.Close();

                // Delete VisualStudioTools regkey
                root = Registry.ClassesRoot;
                root.DeleteSubKey("exefile\\shellex\\ContextMenuHandlers\\VisualStudioTools");

                // Delete Command.com setting
                root = Registry.LocalMachine;
                root.DeleteSubKey("Software\\VisualStudioTools\\Install service");
                root.DeleteSubKey("Software\\VisualStudioTools\\Uninstall service");
                root.DeleteSubKey("Software\\VisualStudioTools");
            }
            catch (Exception e)
            {
                System.Console.WriteLine(e.ToString());
            }
        }
        #endregion
    }
    #endregion
}
